@@ -26,7 +26,7 @@ import re
 
 DESTAR_CFG = "destar_cfg.py"
 CONFIGLETS_DIR = os.getenv('CONFIGLETS_DIR', default='.') 
-
+ASTERISK_MODULES_DIR = os.getenv('ASTERISK_MODULES_DIR', default='/usr/lib/asterisk/modules')
 
 
 
@@ -48,7 +48,7 @@ def loadPythonConfig():
 	global __loaded
 
 	# Forget all config options
-	configlets.config_entries = []
+	configlets.configlet_tree = configlets.ConfigletTree()
 
 	# Try to read destar_cfg.py from Asterisk directory
 	# if this doesn't work, read it from current directory
@@ -63,14 +63,18 @@ def loadPythonConfig():
 				print _("Warning: There is no %s or %s file yet." % (fn,DESTAR_CFG))
 	except NameError:
 		pass
-
+		
 	fixupConfiglets()
-
 	__loaded = True
 
-
-
-
+def fixupConfiglets():
+	configlets.varlist_manager.updateTrunks()
+	configlets.varlist_manager.updateDialouts()
+	for obj in configlets.configlet_tree:
+		obj.createVariables()
+	for obj in configlets.configlet_tree:
+		obj.fixup()
+		obj.createDependencies()
 
 def createPythonConfig(f=None):
 	"""This writes /etc/asterisk/destar_cfg.py.
@@ -82,16 +86,17 @@ def createPythonConfig(f=None):
 		createPythonConfig(sys.stdout)
 	"""
 
-
-	if not configlets.config_entries:
+	if configlets.configlet_tree.empty():
 		return
+	
+	fixupConfiglets()
 
-	if not f:
+	if f is None:
 		f = open(os.path.join(configlets.CONF_DIR,DESTAR_CFG),"w")
 
 	f.write("# -*- coding: iso-latin-1 -*-\n")
 	f.write("# You should execfile() this config\n\n")
-	for c in configlets.config_entries:
+	for c in configlets.configlet_tree:
 		a = c.createPythonConfig()
 		for s in a:
 			f.write("%s\n" % s)
@@ -137,18 +142,12 @@ def initializeAsteriskConfig():
 	c.appendExten("s","Hangup")
 	c.appendExten("i","Hangup")
 
-	c.setSection("phones")
-	c.append("include=apps")
-
 	c = AstConf("sip.conf")
 	c.append("language=%s" % getSetting('language', 'en'))
 	c.append("maxexpirey=3600")
 	c.append("defaultexpirey=3600")
 	c.append("disallow=all")
 
-
-	needModule("res_crypto")
-	needModule("chan_iax2")
 	c = AstConf("iax.conf")
 	c.append("language=%s" % getSetting('language', 'en'))
 
@@ -163,68 +162,82 @@ def initializeAsteriskConfig():
 	c.setSection("general")
 	c.append("language=%s" % getSetting('language', 'en'))
 
+	needModule("pbx_functions")
+	needModule("cdr_sqlite3_custom")
+	c = AstConf("cdr_sqlite3_custom.conf")
+	c.setSection("master")
+	c.append("table => cdr")
+	c.append("columns => clid,src,dst,dcontext,channel,dstchannel,lastapp,lastdata,start,answer,end,duration,billsec,disposition,amaflags,accountcode,uniqueid,userfield,pbx,intrunk,outtrunk,dialout")
+	c.append("values => '${CDR(clid)}','${CDR(src)}','${CDR(dst)}','${CDR(dcontext)}','${CDR(channel)}','${CDR(dstchannel)}','${CDR(lastapp)}','${CDR(lastdata)}','${CDR(start)}','${CDR(answer)}','${CDR(end)}','${CDR(duration)}','${CDR(billsec)}','${CDR(disposition)}','${CDR(amaflags)}','${CDR(accountcode)}','${CDR(uniqueid)}','${CDR(userfield)}','${CDR(pbx)}','${CDR(intrunk)}','${CDR(outtrunk)}','${CDR(dialout)}'")
 
 	c = AstConf("macros.inc")
 	c.append(";")
 	c.append("; Macro to dial a standard local extension")
 	c.append(";")
-	c.append("; format: Macro(stdexten,dest,context,voicebox 1/0)")
+	c.append("; format: Macro(stdexten,dest,voicebox 1/0,ext,pbx)")
 	c.append(";")
 	c.setSection("macro-dial-std-exten")
 	c.append(";")
 	c.append("; test for CFIM (Call Forwarding Immediate)")
-	c.append("exten=s,1,DBget(temp=CFIM/${MACRO_EXTEN})")
-	c.append("exten=s,2,Goto(${ARG2},${temp})")
-
-	c.append("; no CFIM")
-	c.append("exten=s,102,DBGet(temp=DND/${MACRO_EXTEN})")
-	c.append("; no CFIM, DND set")
-	c.append("exten=s,103,GotoIf($[${ARG3}vm = 1vm]?105:104)")
-	c.append("; no CFIM, DND set, but no voicemaiL:")
-	c.append(";TODO: set cause code")
-	c.append("exten=s,104,Hangup")
-	c.append("; no CFIM, DND set, voicemail set:")
-	c.append("exten=s,105,Macro(voicemail,u${MACRO_EXTEN})")
-	c.append("exten=s,106,Hangup")
-
-	c.append("; no CFIM, no DND")
-	c.append("exten=s,203,DBGet(dsec=DSEC/${MACRO_EXTEN})")
-	c.append("exten=s,204,NoOp")
-	c.append("exten=s,205,DBGet(dopt=DOPT/${MACRO_EXTEN})")
-	c.append("exten=s,206,NoOp")
-	c.append("exten=s,207,Dial(${ARG1},${dsec},${dopt})")
-	c.append("; Dial result was 'timeout'")
-	c.append("exten=s,208,SetVar(vmopt=u)")
-	c.append("exten=s,209,Noop")
-	c.append("exten=s,210,DBGet(temp=CFBS/${MACRO_EXTEN})")
-	c.append("exten=s,211,Goto(${ARG2},${temp})")
-
-	c.append("; no CFIM, no DND, no DSEC (dial seconds)")
-	c.append("exten=s,304,SetVar(dsec=45)")
-	c.append("exten=s,305,Goto(205)")
-	c.append("; no CFIM, no DND, no DOPT (dial options)")
-	c.append("exten=s,306,SetVar(dopt=Ttr)")
-	c.append("exten=s,307,Goto(207)")
-
-	c.append("; Dial result was 'unavailable' or 'busy'")
-	c.append("exten=s,308,SetVar(vmopt=b)")
-	c.append("exten=s,309,Noop")
-	c.append("exten=s,310,Goto(210)")
-
-	c.append("; dial did no go throught, CFBS not set")
-	c.append("exten=s,311,GotoIf($[${ARG3}vm = 1vm]?313:312)")
-	c.append("exten=s,312,Hangup")
-	c.append("exten=s,313,Macro(voicemail,${vmopt}${MACRO_EXTEN})")
-
-	c.append("exten=i,0,Hangup")
+	c.append("exten=s,1,Set(fw_ext=${DB(CFIM/${ARG5}/${ARG4})})")
+	c.append('exten=s,n,GotoIf($["${fw_ext}" != ""]?fw)')
 	c.append(";")
-	c.append("; format: Macro(voicemail,<VoiceMail2 arguments>)")
+	c.append("; test for DND (Do Not Disturb)")
+	c.append("exten=s,n,Set(temp=${DB(DND/${ARG5}/${ARG4})})")
+	c.append('exten=s,n,GotoIf($["${temp}" = "True"]?testvmu)')
+	c.append(";")
+	c.append("; get dial seconds")
+	c.append("exten=s,n,Set(dsec=${DB(DSEC/${ARG5}/${ARG4})})")
+	c.append('exten=s,n,GotoIf($["${dsec}" = ""]?:dsecEnd)')
+	c.append("exten=s,n,Set(dsec=45)")
+	c.append("exten=s,n(dsecEnd),NoOp()")
+	c.append(";")
+	c.append("; get dial options")
+	c.append("exten=s,n,Set(dopt=${DIAL_OPTIONS})")
+	c.append('exten=s,n,GotoIf($["${dopt}" = ""]?:doptEnd)')
+	c.append("exten=s,n,Set(dopt=r)")
+	c.append("exten=s,n(doptEnd),NoOp()")
+	c.append(";")
+	c.append("; Dial")
+	c.append("exten=s,n(MainDial),Dial(${ARG1},${dsec},Tt${dopt})")
+	c.append(";")
+	c.append("; Dial result was 'timeout'")
+	c.append("exten=s,n,Set(fw_ext=${DB(CFTO/${ARG5}/${ARG4})})")
+	c.append('exten=s,n,GotoIf($["${fw_ext}" != ""]?fw)')
+	c.append(";")
+	c.append("; no CFTO (Call Forward Timeout), test VM U.")
+	c.append("exten=s,n,Goto(testvmu)")
+	c.append(";")
+	c.append("; Dial result was 'busy'")
+	c.append("exten=s,MainDial+101,Set(fw_ext=${DB(CFBS/${ARG5}/${ARG4})})")
+	c.append('exten=s,n,GotoIf($["${fw_ext}" != ""]?fw)')
+	c.append(";")
+	c.append("; no CFBS (Call Forward Busy), test VM Busy")
+	c.append("exten=s,n,Goto(testvmb)")
+	c.append(";")
+	c.append("exten=s,n(fw),Goto(${ARG2},${fw_ext},1)")
+	c.append(";")
+	c.append('exten=s,n(testvmu),GotoIf($["${ARG3}" = "1"]?:hangup)')
+	c.append('exten=s,n,Set(vmopt=u)')
+	c.append("exten=s,n,Macro(voicemail,${vmopt}${ARG4}@${ARG5})")
+	c.append(";")
+	c.append('exten=s,n(testvmb),GotoIf($["${ARG3}" = "1"]?:hangup)')
+	c.append('exten=s,n,Set(vmopt=b)')
+	c.append("exten=s,n,Macro(voicemail,${vmopt}${ARG4}@${ARG5})")
+	c.append(";")
+	c.append("exten=s,n(hangup),Hangup()")
+	c.append(";")
+	c.append("exten=i,0,Hangup")
+
+	c.append(";")
+	c.append("; format: Macro(voicemail,<VoiceMail arguments>)")
 	c.append(";")
 	c.setSection("macro-voicemail")
 	c.append("exten => s,1,Answer")
-	c.append("exten => s,2,Wait(1)")
-	c.append("exten => s,3,VoiceMail2(${ARG1})")
-	c.append(";exten => s,104,Macro(dial-result,3)")
+	c.append("exten => s,2,Set(TIMEOUT(absolute)=240)")
+	c.append("exten => s,3,Wait(1)")
+	c.append("exten => s,4,VoiceMail(${ARG1})")
+	c.append(";exten => s,105,Macro(dial-result,3)")
 
 	c.append(";exten => h,1,Macro(dial-result)")
 	c.append(";exten => t,1,Macro(dial-result)")
@@ -233,7 +246,7 @@ def initializeAsteriskConfig():
 	c.append("; format: Macro(dial-result,[<cause>])")
 	c.append(";")
 	c.setSection("macro-dial-result")
-	c.append("exten => s,1,AbsoluteTimeout(35)")
+	c.append("exten => s,1,Set(TIMEOUT(absolute)=35)")
 	c.append("exten => s,2,GotoIf($[foo${ARG1} != foo]?cause_${ARG1},1:cause_${HANGUPCAUSE},1)")
 
 	c.append("; undefined error (mostly when an existing extension is currently unavailable)")
@@ -329,54 +342,70 @@ def createAsteriskConfig():
 
 	createPythonConfig()
 	
-	if not configlets.config_entries:
+	if configlets.configlet_tree.empty():
 		return [] 
 	initializeAsteriskConfig()
 	# First write the options
-	for c in configlets.config_entries:
+	for c in configlets.configlet_tree:
 		if isinstance(c, CfgOpt):
 			c.createAsteriskConfig()
 	# Then the rest
-	for c in configlets.config_entries:
+	for c in configlets.configlet_tree:
 		if not isinstance(c, CfgOpt):
 			c.createAsteriskConfig()
 
+		
+	available_modules = []
+	missing_modules = []
+	for f in os.listdir(ASTERISK_MODULES_DIR):
+		if f.endswith('.so'): available_modules.append(f[:-3])
 	c = AstConf("modules.conf")
-	for m in c.modules.preload:
-		c.append("%s.so=yes" % m)
-
 	c.setSection("modules")
 	c.append("autoload=no")
-	for sect in ("pbx", "codec", "format", "res", "cdr", "chan", "app"):
+	for m in c.modules.preload:
+		c.append("preload=%s.so" % m)
+		if not m in available_modules: missing_modules.append(m)
+
+	for sect in ("pbx", "codec", "format", "res", "cdr", "chan", "func", "app"):
 		for m in c.modules[sect]:
 			c.append("load=%s.so" % m)
+			if not m in available_modules: missing_modules.append(m)
 
 	# test if all config files are OK to be written or overwritten
-	res = []
+	tmp_conf = []
 	for fn,cnf in configlets.asterisk_configfiles:
 		# zaptel.conf can't have CONF_TAG at the top, so no need to search for it
-		if fn != 'zaptel.conf' and os.path.exists(cnf.fn):
+		if os.path.exists(cnf.fn):
 			f = open(cnf.fn, "r")
 			s = f.readline()
-			if s != configlets.CONF_TAG:
+			if s != configlets.CONF_TAG or fn == 'zaptel.conf':
 				# Backup file not created by us
 				try:
 					backupAsteriskConfig(fn)
 				except OSError:
 					print _("Error backing up %s") % fn 
-		res.append( (fn,cnf) )
+		tmp_conf.append( (fn,cnf) )
 	
+	res=[]
+	res.append(tmp_conf)
+	res.append(missing_modules)
+
+	return res 
+
+def writeAsteriskConfig():
 	# write all stuff out
+	res = True
 	for _fn,cnf in configlets.asterisk_configfiles:
 		if _fn == 'op_server.cfg' and panelutils.isConfigured() != 1:
 			continue
-		cnf.write()
-	
+		try:
+			cnf.write()
+		except OSError:
+			print _("Error writing up %s") % _fn 
+			res = False
 	return res
 
 
-
-
 def backupAsteriskConfig(fn):
 	"""This backs up one file. It get's a pure filename, say
 	'extensions.conf' and looks into configlets.asterisk_configfiles for
@@ -464,11 +493,13 @@ def configletsGrouped():
 # Some of them are fairly simply, but are here to have very little backend
 # functionality in Config.ptl or other frontend parts.
 #
+# **** TODO: This functions are no longer necessary, and now are used as stub
+#            only
 
 
 def newConfiglet(clazz):
 	"""Create a new configlet with the class name 'clazz'. This new
-	configlet is not added configlist.config_entries, you have to do
+	configlet is not added configlist.configlet_tree, you have to do
 	this manually with addConfiglet()."""
 
 	return globals()[clazz](autoAdd=False)
@@ -477,26 +508,26 @@ def newConfiglet(clazz):
 
 
 def addConfiglet(obj):
-	"""Adds the configlet 'obj' to configlets.config_entries."""
+	"""Adds the configlet 'obj' to configlets.configlet_tree."""
 
 
-	configlets.config_entries.append(obj)
+	configlets.configlet_tree.addConfiglet(obj)
 
 
 
 def updateConfiglet(obj):
-	"""Updates the configlet 'obj' in configlets.config_entries."""
+	"""Updates the configlet 'obj' in configlets.configlet_tree."""
 
 
-	configlets.config_entries[int(obj._id)] = obj
+	configlets.configlet_tree.updateConfiglet(obj)
 
 
 
 def deleteConfiglet(id):
-	"""Deletes the configlet with index 'id' from configlets.config_entries."""
+	"""Deletes the configlet with index 'id' from configlets.configlet_tree."""
 
 
-	del configlets.config_entries[int(id)]
+	configlets.configlet_tree.deleteConfiglet(int(id))
 
 
 
@@ -504,66 +535,25 @@ def deleteConfiglet(id):
 def moveConfigletUp(id):
 	if not __loaded: loadPythonConfig()
 
-	id = int(id)
-	obj = configlets.config_entries[id]
-	id2 = id-1
-	while id2 >= 0:
-		if configlets.config_entries[id2].groupName==obj.groupName:
-			configlets.config_entries[id] = configlets.config_entries[id2]
-			configlets.config_entries[id2] = obj
-			return obj
-		id2 = id2 - 1
-	return None
+	return configlets.configlet_tree.moveConfigletUp(id)
 
 
 
 def moveConfigletDown(id):
 	if not __loaded: loadPythonConfig()
 
-	id = int(id)
-	obj = configlets.config_entries[id]
-	id2 = id+1
-	while id2 < len(configlets.config_entries):
-		if configlets.config_entries[id2].groupName==obj.groupName:
-			configlets.config_entries[id] = configlets.config_entries[id2]
-			configlets.config_entries[id2] = obj
-			return obj
-		id2 = id2 + 1
-	return None
-
-
-
-def fixupConfiglets():
-	"""Calls the fixup() method for all configlets. This should be done
-	whenever the configlets get loaded, added, deleted or modified. This
-	way, configlets can react on the presence of other configlets.
-
-	Example: if a trunk with DID (direct inward dialling) is present,
-	then the DID settings in the phone configuration configlets can be
-	unhidden."""
-
-
-	for obj in configlets.config_entries:
-		obj.fixup()
-	
-
+	return configlets.configlet_tree.moveConfigletDown(id)
 
 def countConfiglets(groupName=None, clazz=None):
 	"Returns the count of all configlets in a given 'group'."
 
 
 	if not __loaded: loadPythonConfig()
-	n = 0
-	for s in configlets.config_entries:
-		#print s.groupName
-		if groupName and s.groupName == groupName:
-			n = n + 1
-		elif clazz and s.__class__.__name__ == clazz:
-			n = n + 1
-	return n
-
-
-
+	if groupName is not None:
+		return len(configlets.configlet_tree[groupName])
+	if clazz is not None:
+		return len(configlets.configlet_tree.getConfigletsByName(clazz))
+	return len(configlets.configlet_tree)
 
 def getConfiglets(group=None, name=None):
 	"""Return a list of all configlets in a given 'group', with a given
@@ -577,21 +567,20 @@ def getConfiglets(group=None, name=None):
 
 
 	if not __loaded: loadPythonConfig()
-	a = []
-	n = 0
-	for obj in configlets.config_entries:
-		obj._id = n
-		n = n + 1
-		if obj.groupName == group:
-			a.append(obj)
-		if obj.shortName==name or obj.__class__.__name__==name:
-			a.append(obj)
-	return a
+	
+	result = []
+	
+	if group is not None: 
+		result += configlets.configlet_tree.getConfigletsByGroup(group)
+	if name is not None:
+		result += configlets.configlet_tree.getConfigletsByName(name)
+	return result
 
 
 
+# TODO: examine the real utility of this three next functions
 
-def getConfiglet(id=None, name=None):
+def getConfiglet(_id=None, name=None):
 	"""This returns a configlet by the 'id'. The id is not set
 	in stone, but is only valid from one getConfiglets() call
 	to the next."""
@@ -599,59 +588,42 @@ def getConfiglet(id=None, name=None):
 
 	if not __loaded: loadPythonConfig()
 
-	if name:
-		n = 0
-		for obj in configlets.config_entries:
-			try:
-				if obj.name == name:
-					obj._id = n
-					return obj
-			except AttributeError:
-				pass
-			n = n + 1
-		return None
+	if name is not None:
+		return configlets.configlet_tree.getConfigletByName(name)
+	if _id is not None:
+		return configlets.configlet_tree.getConfiglet(_id)
 
-	# If we have no name and no id, we can't do better:
-	if id==None:
-		return None
-
-	try:
-		obj = configlets.config_entries[int(id)]
-		obj._id = id
-		return obj
-	except:
-		return None
 
 # Configlets can't import the Backend (because the Backend loads/imports
 # the configlets. So we make this method manually known in the configlets module.
 configlets.getConfiglet = getConfiglet
 
 
-
-
+# FIXME: What an ugly function!!!
 def getConfig(clazz, name, default=None):
 	"""This searches for the first found configlet with the class
 	'clazz'. If found, it looks if this configlet has the attribute name
 	and returns it value or some default."""
 
-	for obj in configlets.config_entries:
-		if obj.__class__.__name__==clazz:
-			return getattr(obj, name, default)
+	obj = configlets.configlet_tree.getConfigletsByName(clazz)
+	if obj is not None and len(obj) > 0:
+		obj = obj[0]
+		return getattr(obj, name, default)
 	return default
+
 # Configlets can't import the Backend (because the Backend loads/imports
 # the configlets. So we make this method manually known in the configlets module.
 configlets.getConfig = getConfig
 
-
-
-
-def getChoice(clazz, key='name',val='name'):
+# FIXME: here is a mix of backend and frontend stuff, needs to be refactored
+# FIXME: this is an ugly function too
+def getChoice(clazz, key='name',val='name',sort=True):
 	"""This is used to generate a list of tuples which we later use
 	in the select widgets of type "choice" or "mchoice".
 
 	'clazz' is the classname the configlet must have."""
 
-
+	# TODO: this sould be changed for something like CheckConfig
 	if not __loaded:
 		loadPythonConfig()
 	a = []
@@ -660,13 +632,16 @@ def getChoice(clazz, key='name',val='name'):
 		obj2 = globals()[clazz]
 	except KeyError:
 		return a
-	for obj in configlets.config_entries:
-		obj._id = n
-		n = n + 1
-		if isinstance(obj, obj2):
-			a.append( (obj.__dict__[val], obj.__dict__[key]) )
+
+	for obj in configlets.configlet_tree.getConfigletsByClass(obj2):
+		a.append( (obj.__dict__[val], obj.__dict__[key]) )
+		
 	if a == [] :
 		a.append( ('','') )
+		
+	if sort:
+		a.sort()
+
 	return a
 
 configlets.__getChoice = getChoice
@@ -708,7 +683,7 @@ def determineStateOfPhones():
 
 	phones = []
 	other  = []
-	for p in configlets.config_entries:
+	for p in configlets.configlet_tree:
 		try:
 			chan = p.channelString()
 		except AttributeError:
@@ -780,6 +755,7 @@ for f in os.listdir(CONFIGLETS_DIR):
 
 
 
+
 ####################################################################################
 #
 # Test code if this file is called with 'python backend.py':
@@ -795,8 +771,14 @@ def reloadAsterisk():
 	"""This reloads the Asterisk PBX."""
 	createPythonConfig()
 	createAsteriskConfig()
+	writeAsteriskConfig()
 	import manager
 	s = manager.reloadAsterisk()
 	if panelutils.isConfigured():
 		s += panelutils.restartPanelDaemon()
 	return "<br/>".join(s)
+
+def createDocs():
+	for c in configletsList():
+		cfg = globals()[c.__name__](autoAdd=False)
+		cfg.writeDoc()
