@@ -24,15 +24,24 @@ from configlets import *
 class CfgAppMilliwatt(CfgApp):
 
 	shortName   = _("Milliwatt test")
+	newObjectTitle  = _("New milliwatt test extension")
 	description = _("""Generate a Constant 1000Hz tone at 0dbm (mu-law). Used for
 			measuring.""")
-	variables   = [VarType("ext", title=_("Extension"), len=6)]
+			
+	def createVariables(self):
+		self.variables   = [	VarType("pbx",    title=_("Virtual PBX"), type="choice", options=getChoice("CfgOptPBX")),
+					VarType("ext", title=_("Extension"), len=6)]
+		self.dependencies = [ DepType("pbx", 
+					type="hard",
+					message = _("This is a Dependency")),
+					]
+		
 
 	def createAsteriskConfig(self):
 		needModule("app_milliwatt")
 
 		c = AstConf("extensions.conf")
-		c.setSection("apps")
+		c.setSection(self.pbx)
 		c.appendExten(self.ext, "Answer")
 		c.appendExten(self.ext, "Milliwatt")
 		c.appendExten(self.ext, "Hangup")
