@@ -24,14 +24,22 @@ from configlets import *
 class CfgAppMusic(CfgApp):
 
 	shortName   = _("Listen to music-on-hold")
+	newObjectTitle  = _("New music-on-hold extension")
 	description = _("Play Play Music On Hold until you hang up.")
-	variables   = [ VarType("ext", title=_("Extension"), len=6),
+	
+	def createVariables(self):
+		self.variables   = [ VarType("pbx",    title=_("Virtual PBX"), type="choice", options=getChoice("CfgOptPBX")),
+			VarType("ext", title=_("Extension"), len=6),
 			VarType("moh",	  title=_("Music-on-hold class"), type="choice", optional=True,
 			options=getChoice("CfgOptMusic"))]
+		self.dependencies = [ DepType("pbx", 
+					type="hard",
+					message = _("This is a Dependency")),
+					]
 
 	def createAsteriskConfig(self):
 		c = AstConf("extensions.conf")
-		c.setSection("apps")
+		c.setSection(self.pbx)
 		c.appendExten(self.ext, "Answer")
 		c.appendExten(self.ext, "Wait(1)")
 		if self.moh:
