@@ -24,7 +24,10 @@ from configlets import *
 class CfgOptEnum(CfgOpt):
 
 	shortName = _("ENUM search domain")
-	variables = [VarType("search", title=_("Domain to search for ENUM entries"))]
+	newObjectTitle= _("New ENUM search domain")
+	
+	def createVariables(self):
+		self.variables = [VarType("search", title=_("Domain to search for ENUM entries"))]
 
 	def createAsteriskConfig(self):
 		c = AstConf("enum.conf")
