@@ -24,14 +24,27 @@ from configlets import *
 class CfgOptMusic(CfgOpt):
 
 	shortName = _("Background music directory")
-	variables = [VarType("name", title=_("Name"), len=15),
-		     VarType("type", title=_("Type"), type="choice", options=("mp3", "quietmp3","mp3nb","quietmp3nb")),
-		     VarType("dir",  title=_("Directory"))]
+	newObjectTitle= _("New background music directory")
+	
+	def createVariables(self):
+		self.variables = [VarType("name", title=_("Name"), len=15),
+		     VarType("type", title=_("Type"), type="choice", options=("mp3", "quietmp3","mp3nb","quietmp3nb","custom","files")),
+		     VarType("dir",  title=_("Directory"), len=255),
+		     VarType("app",  title=_("Application"), len=255, optional=True),
+				]
+
+	def checkConfig(self):
+		return CfgOpt.checkConfig(self)
+		if self.type == "custom" and not self.app:
+			return ("app",_("Please specify an application"))
 
 	def createAsteriskConfig(self):
 		c = AstConf("musiconhold.conf")
-		c.setSection("classes")
-		c.append("%s=%s:%s" % (self.name, self.type, self.dir))
+		c.setSection(self.name)
+		c.append("mode=%s" % self.type)
+		c.append("directory=%s" % self.dir)
+		if self.type == "custom" and self.app:
+			c.append("application=%s" % self.app)
 
 	def row(self):
 		return (self.shortName, self.name)
