@@ -9,12 +9,12 @@
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 #
 
 
@@ -22,7 +22,9 @@ import sys, os, types, sha, binascii, time
 import panelutils
 
 CONF_DIR = "/etc/asterisk"
+DOC_DIR = os.getenv('DESTAR_DOC_DIR', default='/tmp/destar-doc')
 CONF_TAG = "; Automatically created by DESTAR\n"
+ASTERISK_MODULES_DIR = os.getenv('ASTERISK_MODULES_DIR', default='/usr/lib/asterisk/modules') 
 
 
 class AsteriskConfigFile:
@@ -47,10 +49,10 @@ class AsteriskConfigFile:
 			fn = os.path.join(panelutils.PANEL_CONF_DIR, fn)
 		elif fn.find('/')==-1:
 			fn = os.path.join(CONF_DIR, fn)
-		self.fn       = fn
+		self.fn		  = fn
 		self.sections = {}		# dictionary of sections
-		self.order    = []		# ordered list of sections
-		self.dirty    = False		# nothing to write yet
+		self.order	  = []		# ordered list of sections
+		self.dirty	  = False		# nothing to write yet
 		self.extpriority = 1
 		self.section  = "general"
 		self.destar_comment = True
@@ -71,6 +73,7 @@ class AsteriskConfigFile:
 					f.write(l)
 					f.write("\n")
 			self.dirty = False
+	
 
 	def setSection(self, sect):
 		"Select a config file section where append() etc will put their data"
@@ -146,7 +149,7 @@ class AsteriskConfigFile:
 				self.append("%s=no" % name)
 		else:
 			print "invalid type ", val
-			raise Error
+			raise Exception
 
 
 asterisk_configfiles = []
@@ -189,90 +192,192 @@ generatePassword = __PasswordGeneratorClass()
 #########################################################################
 
 
-def needModule(mod):
-	#print "Need module", mod
+def needModule(mod,preload=False):
 	c = AstConf("modules.conf")
 	if not "modules" in dir(c):
-		#print "setting c.modules"
 		c.modules = Holder(
 			pbx =	[
 				"pbx_config",
 				"pbx_spool",
+				"pbx_functions",
 				],
-			# TODO: get list of codecs and format from /usr/lib/asterisk/modules
 			codec = [
-				"codec_a_mu",
-				"codec_adpcm",
-				"codec_alaw",
-				"codec_g726",
-				"codec_gsm",
-				"codec_ilbc",
-				"codec_lpc10",
-				"codec_ulaw",
 				],
 			format = [
-				"format_g726",
-				"format_g729",
-				"format_gsm",
-				"format_h263",
-				"format_ilbc",
-				"format_jpeg",
-				"format_pcm",
-				"format_pcm_alaw",
-				"format_vox",
-				"format_wav",
-				"format_wav_gsm",
 				],
-			res =   [ "res_musiconhold",
+			res =	[ "res_musiconhold",
 				  "res_features" ],
-			cdr =   [ ],
+			cdr =	[	
+				  "cdr_sqlite3_custom",
+				  "cdr_csv", #To be used as a backup
+				],
 			chan =	[ ],
+			func =	[ 
+				"func_callerid",
+				],
 			app =	[
 				"app_db",
-				"app_dial",	# needs res_musiconhold, res_parking
+				"app_dial", # needs res_musiconhold, res_parking
 				"app_macro",
 				"app_playback",
+				"app_cdr",
 				],
 			preload = [],
 			)
-
-	sect = mod.split("_")[0]
-	#print "sect", sect
-	try:
-		sect = c.modules[sect]
-		#print "sect now:", sect
-	except KeyError:
-		# create new array
-		c.modules[sect] = [mod]
-		return
+		# Get list of codecs and formats from ASTERISK_MODULES_DIR
+		for f in os.listdir(ASTERISK_MODULES_DIR):
+			if f.startswith('codec_'): c.modules["codec"].append(f[:-3])
+			if f.startswith('format_'): c.modules["format"].append(f[:-3])
+	
+	if preload:
+		sect = c.modules["preload"]
+	else: 
+		sect = mod.split("_")[0]
+		try:
+			sect = c.modules[sect]
+		except KeyError:
+			# create new array
+			c.modules[sect] = [mod]
+			sect = c.modules[sect]
+			return
 	# sect contains now the entire section array
 	if not mod in sect:
 		sect.append(mod)
-	#print "sect finally", sect
 
 
 
 #########################################################################
 
-context_entries = []
+class ConfigletTree:
+	
+	def __init__(self):
+		self.groups = {}
+		self.name = ''
+	
+	def __getitem__(self, key):
+		if self.groups.has_key(key):
+			return self.groups[key]
+		else: return []
 
-def useContext(ctx):
-	"""Use this function to remember all contexts that are in use."""
+	def __setitem__(self, key, val):
+		self.groups[key] = val
+		
+	def __iter__(self):
+		for group in self.groups.values():
+			for element in group:
+				yield element
+				
+	def __str__(self):
+		stringname = ''
+		for element in self:
+			stringname += str(element) + ', '
+		return stringname[:-2]
+	
+	def __len__(self):
+		length = 0
+		for i in self.groups.values():
+			length += len(i)
+		return length
+			
+	def updateConfiglet(self, obj):
+		configlet_list = self.groups[obj.groupName]
+		
+		
+		for i in range(len(configlet_list)):
+			if configlet_list[i]._id == obj._id:
+				configlet_list[i] = obj
+				
+	def deleteConfiglet(self, _id):
+		for group in self.groups.values():
+			for i in range(len(group)):
+				if group[i]._id == _id:
+					for dep in group[i].dependent_objs:
+						self.deleteConfiglet(dep.configlet._id)
+					del group[i]
+					return
+					
+	def moveConfigletUp(self, _id):
+		obj = self.getConfiglet(_id)
+		configlet_list = self.groups[obj.groupName]
+		
+		for i in range(len(configlet_list)):
+			if configlet_list[i]._id == obj._id and i > 0:
+				configlet_list[i] = configlet_list[i-1]
+				configlet_list[i-1] = obj
+				return obj
+		return None
+		
+	def moveConfigletDown(self, _id):
+		obj = self.getConfiglet(_id)
+		configlet_list = self.groups[obj.groupName]
+		
+		for i in range(len(configlet_list)):
+			if configlet_list[i]._id == obj._id and i < (len(configlet_list) -1):
+				configlet_list[i] = configlet_list[i+1]
+				configlet_list[i+1] = obj
+				return obj
+		return None
+	
+	def getConfigletsByGroup(self, groupName):
+		return self[groupName]
+	
+	#TODO: Make this as fast as getConfigletsByGroup
+	def getConfigletsByName(self, name):
+		result = []
+		for obj in self:
+			if obj.shortName == name or obj.__class__.__name__ == name:
+				result.append(obj)
+		return result
 
-	if ctx not in context_entries:
-		context_entries.append(ctx)
-	#print context_entries
+	def getConfigletsByClass(self, _class):
+		result = []
+		for obj in self:
+			if isinstance(obj, _class):
+				result.append(obj)
+		return result
+		
+	def hasConfiglet(self, _class):
+		for obj in self:
+			if obj.__class__.__name__ == _class: 
+				return True
+		return False
+			
 
+	def empty(self):
+		return len(self)==0
+		
+	def getConfiglet(self, _id):
+		for obj in self:
+			try:
+				if obj._id == int(_id):
+					return obj
+			except:
+				pass
+		return None
+		
+	def getConfigletByName(self, name):
+		for obj in self:
+			try:
+				if obj.name == name:
+					return obj
+			except:
+				pass
+		return None
+		
+	def addConfiglet(self, configlet):
+		if not self.groups.has_key(configlet.groupName):
+			self.groups[configlet.groupName] = []
+		self.groups[configlet.groupName].append(configlet)
+	
 
-
-#########################################################################
+configlet_tree = ConfigletTree()
 
 class Holder(object):
 	"""
 	This is a simple wrapper class so that you can write
 
 	 foo = Holder(bar = 1,
-	            baz = "test")
+				baz = "test")
 	instead of
 
 	 foo["bar"] = 1
@@ -301,7 +406,14 @@ class Holder(object):
 		return "<configlets.Holder object: " + self.__dict__.__repr__()
 
 
-
+class DepType(Holder):
+	"""We use this class to store meta-information about dependendencies"""
+	
+	def __init__(self,name,**kw):
+		Holder.__init__(self,**kw)
+		self.name = name
+		self.__dict__.setdefault("type","hard")
+		self.__dict__.setdefault("message", _("This is a Dependency"))
 
 class VarType(Holder):
 	"""We use this class to store meta-information about configuration variables"""
@@ -324,7 +436,11 @@ class VarType(Holder):
 
 
 
-config_entries = []
+
+class DependentObject:
+	def __init__(self, configlet, dependency):
+		self.configlet = configlet
+		self.dependency = dependency
 
 class Cfg(Holder):
 	"""Base class for all configlets.
@@ -343,22 +459,27 @@ class Cfg(Holder):
 	(to keep 'name' available for actual data)."""
 
 	#shortName = "Cfg class (do not use directly)"
-	#group     = "Generic option"
+	#group	   = "Generic option"
 	variables  = []
+	dependent_objs = []
+	dependencies = []
 
 
 	def __init__(self,autoAdd=True,**kw):
 		"""Stores values from '*kw' into self.__dict__ and add the
-		newly instantiated object into configlets.config_entries if
+		newly instantiated object into configlets.configlet_tree if
 		'autoAdd' says so."""
 
 		Holder.__init__(self,**kw)
 
 		# Store the object into global array
-		global config_entries
+		global configlet_tree
+		self._id = len(configlet_tree)
 		if autoAdd:
-			self._id = len(config_entries)
-			config_entries.append(self)
+			configlet_tree.addConfiglet(self)
+		else:
+			self.createVariables();
+			self.fixup();
 
 		for v in self.variables:
 			# Labels don't have values. We set 'optional' to True
@@ -367,24 +488,52 @@ class Cfg(Holder):
 			if v.type=="label":
 				v.optional = True
 				continue
+				
+		self.dependent_objs = []
+		self.dependencies = []
+				
+	def __getattr__(self, field):
+		self.createVariables()
+		self.fixup()
+		if self.__dict__.has_key(field):
+			return self.__dict__[field]
+		else:
+			raise AttributeError
 
-			# When we have a default value, then store the value
-			# but don't overwrite an old value
-			if v.__dict__.has_key("default") and v.default:
-				try:
-					if not self.__dict__[v.name]:
-						self.__dict__[v.name] = v.default
-				except:
-					self.__dict__[v.name] = v.default
+	def lookPanel(self):
+		if panelutils.isConfigured() == 1:
+			for v in self.variables:
+				if v.name == "panelLab" or v.name == "panel":
+					v.hide = False
+					
+	def renameDependencies(self, new_name):
+		for dep in self.dependent_objs:
+				dep.configlet.__dict__[dep.dependency.name] = new_name
 
+	def hasDependencies(self):
+		return len(self.dependent_objs) > 0
+
+	def createDependencies(self):
+		for dep in self.dependencies:
+			if self.__dict__.has_key(dep.name):
+				obj_name = self.__dict__[dep.name]
+				import configlets
+				obj = configlets.configlet_tree.getConfigletByName(obj_name)
+				if obj is None:
+					return
+				dependent_obj = DependentObject(self, dep)
+				obj.dependent_objs.append(dependent_obj)
+
+	def createVariables(self):
+		variables = []
+		
+	def clearDependencies(self):
+		self.dependent_objs = []
 
 	def fixup(self):
 		"""Each configlet's fixup() method get's called after the
 		modules have been loaded from the config file."""
 
-		global context_entries
-		context_entries = []
-
 		# Make sure all variables are set:
 		for v in self.variables:
 			if not self.__dict__.has_key(v.name):
@@ -446,13 +595,14 @@ class Cfg(Holder):
 		"""Test if all variables are set, used by createAsteriskConfig().
 		May be overridden for additional test."""
 
-		# Make sure we don't add two thingies with the same extension
+		# Make sure we don't add two thingies with the same extension on the same pbx
+		global configlet_tree
 		if self.__dict__.has_key('ext'):
-			for o in config_entries:
+			for o in configlet_tree:
 				if o==self: continue
 				try:
-					if o.ext == self.ext:
-						return ("ext", _("Extension already in use"))
+					if o.ext == self.ext and o.pbx == self.pbx:
+						return ("ext", _("Extension already in use on that PBX"))
 				except AttributeError:
 					pass
 
@@ -460,7 +610,7 @@ class Cfg(Holder):
 		if self.__dict__.has_key('name'):
 			if self.name.find("-") > 1 or self.name.find(" ") > 1:
 				return ("name", _("Name should not contain '-' or spaces"))
-			for o in config_entries:
+			for o in configlet_tree:
 				if o==self: continue
 				try:
 					if o.name == self.name:
@@ -483,20 +633,30 @@ class Cfg(Holder):
 
 		python_cfg = []
 		python_cfg.append("%s(" % self.__class__.__name__)
+
 		for v in self.variables:
 			if not self.__dict__.has_key(v.name): continue
 			_v = self.__dict__[v.name]
-			if _v == None:
+			if _v == None or _v == "":
 				continue
 			#print v.name,v.type,_v
+			if v.__dict__.has_key("default") and _v==v.default:
+				continue
 			if v.type in ("string","rostring","choice","mchoice","radio"):
 				cont = '"%s"' % _v
 			elif v.type=="text":
 				cont = '"""%s"""' % _v
 			elif v.type=="int":
-				cont = _v
+				if _v == 0:
+					print v.name
+					continue
+				else:
+					cont = _v
 			elif v.type=="bool":
-				cont = ("False","True")[_v]
+				if _v:
+					cont = "True"
+				else:
+					continue
 			elif v.type=="label":
 				continue
 			else:
@@ -507,6 +667,46 @@ class Cfg(Holder):
 		python_cfg.append("")
 		return python_cfg
 
+	def writeDoc(self, d=None):
+		"Write my documentation"
+		if not d:
+			d = DOC_DIR
+		fn = os.path.join(d, "%s.xml" % self.__class__.__name__.lower())
+		f = open(fn, "w")
+		f.write('<?xml version="1.0" encoding="UTF-8"?>')
+		f.write("\n")
+		f.write("<!-- file: autogenerated/%s.xml -->" % self.__class__.__name__.lower())
+		f.write("\n")
+		f.write("<!-- File automatically created by DESTAR -->")
+		f.write("\n")
+		f.write('<!-- xml-parent-document: "../configlets.xml" -->')
+		f.write("\n")
+		f.write("<!-- Destar Manual -->")
+		f.write("\n")
+		f.write("<!-- Distributed under the same Destar license  -->")
+		f.write("\n\n")
+		f.write('<sect2 id="%s">\n\n' % self.__class__.__name__.lower())
+		f.write("<title>%s - %s</title>\n\n" % (self.groupName,self.shortName))
+
+		try:
+			if self.description:
+				f.write("<para>%s</para>\n\n" % self.description)
+		except AttributeError:
+			print _("The configlet '%s' does not have a description" % self.__class__.__name__)
+		
+		f.write("<para>Form fields:</para>\n\n<itemizedlist>\n\n")
+		for var in self.variables:
+			if var.title:
+				f.write("  <listitem>\n    <para>\n")
+				if var.type == "label":
+					f.write("Section [%s]" % var.title)
+				else:
+					f.write("%s" % var.title)
+					if var.hint:
+						f.write(": %s" % var.hint)
+				f.write("\n    </para>\n  </listitem>\n")
+		f.write("</itemizedlist>\n\n")
+		f.write("</sect2>")
 	
 
 #######################################################################
@@ -523,7 +723,7 @@ def getChoice(clazz):
 	Here we actually use a trick. In the configlets, the variables[]
 	get's build once at configlets load-time. While the configlets are
 	in the middle of being loaded, you can't really iterate in
-	configlets.config_entries to get a list of, say, all Phones. So this
+	configlets.configlet_tree to get a list of, say, all Phones. So this
 	iteration needs to be postponed. We do this with a lambda function.
 
 	This getChoice() uses __getChoice(), which is not defined here in
@@ -569,23 +769,57 @@ class CfgOptSingle(CfgOpt):
 	def isAddable(self, clazz=None):
 		"""Allow Options to be added exactly once. We'll enforce this
 		by looking if the current class is already contained in
-		config_entries[].
+		configlet_tree.
 
-		If  a child class, e.g.  CfgOptZapAudio.isAddable(), wants
+		If	a child class, e.g.	 CfgOptZapAudio.isAddable(), wants
 		to call us, then 'self' is no longer CfgOptZapAudio, but it
 		is CfgOptSingle. This is because of the classMethod
 		attribute of isAddable. So we have an optional parameter
 		'clazz' where the child class can tell us which class should
-		be unique in the config_entries."""
+		be unique in the configlet_tree."""
 
+		global configlet_tree
 		if not clazz:
 			clazz = self
-		for o in config_entries:
+		for o in configlet_tree:
 			#print o.__class__, clazz
 			if o.__class__ == clazz: return False
 		return True
 	isAddable = classmethod(isAddable)
 
+class VarListManager:
+
+	def __init__(self):
+		self.trunks = []
+		self.dialouts = []
+		
+	def hasTrunks(self):
+		return len(self.trunks)>0
+		
+	def getTrunks(self):
+		return self.trunks
+		
+	def updateTrunks(self):
+		self.trunks = []
+		global configlet_tree
+		for obj in configlet_tree['Trunks']:
+			self.trunks.append(VarType("trunk_%s" % obj.name, title=_("%s") % obj.name, type="bool", optional=True,render_br=False))
+			self.trunks.append(VarType("trunk_%s_price" % obj.name, title=_("Price/Account_code for this pattern"), type="int", optional=True, len=10, default=0))
+
+	def hasDialouts(self):
+		return len(self.dialouts)>0
+
+	def getDialouts(self):
+		return self.dialouts
+
+	def updateDialouts(self):
+		self.dialouts = []
+		global configlet_tree
+		for obj in configlet_tree['Dialout']:
+			self.dialouts.append(VarType("dialout_%s" % obj.name, title=_("%s") % obj.name, type="bool", optional=True,render_br=False))
+			self.dialouts.append(VarType("dialout_%s_secret" % obj.name, title=_("Password:"), len=50, optional=True))
+
+varlist_manager = VarListManager()
 
 
 class CfgTrunk(Cfg):
@@ -597,6 +831,10 @@ class CfgTrunk(Cfg):
 	def __init__(self,**kw):
 		Cfg.__init__(self,**kw)
 
+	def fixup(self):
+		Cfg.fixup(self)
+		self.lookPanel()
+
 
 	def head(self):
 		return (_("Name"), _("Type"), _("Dial Command"))
@@ -612,38 +850,55 @@ class CfgTrunk(Cfg):
 	def isAddable(self):
 		"We can only add this object if we have at least one other phone defined."
 
-		# BUG: it does somehow not work to simply write for obj in config_entries,
+		# BUG: it does somehow not work to simply write for obj in configlet_tree,
 		# despite the "from configlets import *" above
-		global config_entries
-		for obj in config_entries:
-			if obj.groupName == 'Phones':
-				return True
-		return False
+		global configlet_tree
+		if len(configlet_tree['Phones']) > 0:
+			return True
+		else:
+			return False
 	isAddable = classmethod(isAddable)
 	# BUG: if the choosed phone is deleted, we have a problem
 
 	def checkConfig(self):
-                res = Cfg.checkConfig(self)
-                if res:
-                        return res
+		res = Cfg.checkConfig(self)
+		if res:
+			return res
 		if self.contextin == 'phone' and not self.phone:
 			return ('phone',_("You should select a phone to ring to"))
 
-	def fixup(self):
-		Cfg.fixup(self)
-		if panelutils.isConfigured() == 1:
-			for v in self.variables:
-				if v.name == "panelLab" or v.name == "panel":
-					v.hide = False
-		
 	def createIncomingContext(self): 
 		c = AstConf("extensions.conf")
 		contextin = "in-%s" % self.name
 		c.setSection(contextin)
+		c.appendExten("_X.","Set(CDR(intrunk)=%s)" %  self.name)
+		if self.clid:
+			needModule("func_callerid")
+			c.appendExten("_X.","Set(CALLERID(name)=%s)" %  self.clid)
+		global configlet_tree
 		if self.contextin == 'phone' and self.phone:
-			c.appendExten("s", "Goto(phones,%s,1)" % self.phone)
+			obj = configlet_tree.getConfigletByName(self.phone)
+			try:
+				pbx = obj.pbx
+				c.appendExten("_X.", "Goto(%s,%s,1)" % (pbx,self.phone))
+			except AttributeError:
+				pass
+		if self.contextin == 'ivr' and self.ivr:
+			c.appendExten("_X.", "Goto(%s,s,1)" % self.ivr)
+		c.appendExten("s","Set(CDR(intrunk)=%s)" %  self.name)
+		if self.clid:
+			needModule("func_callerid")
+			c.appendExten("s","Set(CALLERID(name)=%s)" %  self.clid)
+		if self.contextin == 'phone' and self.phone:
+			obj = configlet_tree.getConfigletByName(self.phone)
+			try:
+				pbx = obj.pbx
+				c.appendExten("s", "Goto(%s,%s,1)" % (pbx,self.phone))
+			except AttributeError:
+				pass
 		if self.contextin == 'ivr' and self.ivr:
 			c.appendExten("s", "Goto(%s,s,1)" % self.ivr)
+
 		
 class CfgPhone(Cfg):
 	"""Base class for all phone devices."""
@@ -657,15 +912,17 @@ class CfgPhone(Cfg):
 
 
 	def head(self):
-		return (_("Type"), _("Extension"), _("Name"))
+		return (_("Type"), _("Extension"), _("Name"), _("Virtual PBX"))
 
 
 	def row(self):
 		try:
 			ext = self.ext
+			pbx = self.pbx
 		except AttributeError:
 			ext = _('None')
-		return (self.shortName, ext, self.name)
+			pbx = _('None')
+		return (self.shortName, ext, self.name, pbx)
 
 
 	def channelString(self):
@@ -674,67 +931,55 @@ class CfgPhone(Cfg):
 
 	def fixup(self):
 		Cfg.fixup(self)
-		useContext("phones")
-
-		if panelutils.isConfigured() == 1:
-			for v in self.variables:
-				if v.name == "panelLab" or v.name == "panel":
-					v.hide = False
-
-		global config_entries
-		dialouts=False
-		for obj in config_entries:
-			if obj.groupName == 'Dialout':
-				dialouts=True
-				alreadyappended = False
-				for v in self.variables:	
-					if v.name == "dialout_"+obj.name:
-						alreadyappended = True
-				if not alreadyappended:
-					self.variables.append(VarType("dialout_%s" % obj.name, title=_("%s") % obj.name, type="bool", optional=True,render_br=False))
-					self.variables.append(VarType("dialout_%s_secret" % obj.name, title=_("Password:"), len=50, optional=True))
-		if dialouts:
-			for v in self.variables:
-				if v.name == "Dialout" or v.name=="timeout":
-					v.hide = False
-	
-		queues=False
-		for obj in config_entries:
-			if obj.__class__.__name__ == 'CfgPhoneQueue':
-				queues=True
-		if queues:
+		self.lookPanel()
+		global configlet_tree
+		if configlet_tree.hasConfiglet('CfgPhoneQueue'):
 			for v in self.variables:
 				if v.name == "QueueLab" or v.name == "queues":
 					v.hide = False
 
-
-	def createDialEntry(self, extensions, exten):
-		ret = extensions.appendExten(exten, "Macro(dial-std-exten,%s/%s,%s,%d)" % (
+	def createDialEntry(self, extensions, exten, pbx, ext):
+		ret = extensions.appendExten(exten, "Macro(dial-std-exten,%s/%s,out-%s,%d,%s,%s)" % (
 			self.technology,
 			self.name,
-			"phones",
-			int(self.usevm))
-		      )
+			self.name,
+			int(self.usevm),
+			ext,
+			pbx
+			))
 
 	def createExtensionConfig(self):
 		needModule("res_adsi")
 		needModule("app_voicemail")
 		extensions = AstConf("extensions.conf")
-		extensions.setSection("phones")
-		if self.ext:
-			self.createDialEntry(extensions, self.ext)
-		self.createDialEntry(extensions, self.name);
+                try:
+                        pbx = self.pbx
+                except AttributeError:
+                        pbx = "phones"
+                extensions.setSection(pbx)
+		extensions.appendExten(self.ext,"Set(CDR(pbx)=%s,CDR(userfield)=%s)" % (pbx,self.name))
+		self.createDialEntry(extensions, self.ext, pbx, self.ext)
+		extensions.appendExten(self.name,"Set(CDR(pbx)=%s,CDR(userfield)=%s)" % (pbx,self.name))
+		self.createDialEntry(extensions, self.name, pbx, self.ext)
 
+	def createHintConfig(self):
+		extensions = AstConf("extensions.conf")
+        	try:
+            		pbx = self.pbx
+        	except AttributeError:
+            		pbx = "phones"
+        	extensions.setSection(pbx)
+		extensions.append("exten=%s,hint,%s/%s" % (self.ext, self.technology, self.name))
 
 	def createVoicemailConfig(self, conf):
 		if self.ext and self.usevm:
 			needModule("res_adsi")
 			needModule("app_voicemail")
 			if self.usemwi:
-				conf.append("mailbox=%s@default" % self.ext)
+				conf.append("mailbox=%s@%s" % (self.ext,self.pbx))
 
 			vm = AstConf("voicemail.conf")
-			vm.setSection("default")
+			vm.setSection(self.pbx)
 			try:
 				pin = self.pin
 			except:
@@ -742,7 +987,10 @@ class CfgPhone(Cfg):
 			#TODO: deal with timezones
 			#options = "tz=cest"
 			options = ""
-			vm.append("%s=%s,%s,,%s" % (self.ext, pin, self.name, options))
+			if self.email:
+				vm.append("%s=%s,%s,%s,%s" % (self.ext, pin, self.name, self.email, options))
+			else:
+				vm.append("%s=%s,%s,,%s" % (self.ext, pin, self.name, options))
 	
 	def createQueuesConfig(self):
 		try:
@@ -757,23 +1005,54 @@ class CfgPhone(Cfg):
 	def createOutgoingContext(self):
 		c = AstConf("extensions.conf")
 		c.setSection("out-%s" % self.name)
-		c.append("include=>phones")
+		try:
+			pbx = self.pbx
+		except AttributeError:
+			pbx = "phones"
+		c.append("include=>%s" % pbx)
+		c.appendExten("i","Playback(privacy-invalid)")
 		try:
 			timeoutvalue = not self.timeout and "0" or "1"
 		except AttributeError:
 			timeoutvalue=0
-		global config_entries
-		for obj in config_entries:
+		global configlet_tree
+		for obj in configlet_tree:
 			if obj.__class__.__name__ == 'CfgDialoutNormal':
 				try:
 					if self.__getitem__("dialout_"+obj.name):
-						secret = self.__getitem__("dialout_%s_secret" % obj.name)
+						c.appendExten("%s" % obj.pattern,"Set(CDR(pbx)=%s)" % (self.pbx))
+						c.appendExten("%s" % obj.pattern,"Set(CDR(userfield)=%s)" % (self.name))
+						c.appendExten("%s" % obj.pattern,"Set(CDR(dialout)=%s)" % (obj.name))
+						if self.calleridnum:
+							c.appendExten("%s" % obj.pattern,"Set(CALLERID(number)=%s)" % self.calleridnum)
+							
+						if self.monitor:
+							needModule("app_mixmonitor")	
+							options = ""
+							if self.monitorappend:
+								options = 'a' 
+							if self.monitorwhenbridged:
+								options = options+'b'
+							if self.heardvol == self.spokenvol:
+								options = options+'W(%s)' % (self.heardvol)
+							else:          
+								options = options+'v(%s)V(%s)' % (self.heardvol, self.spokenvol)        
+							if self.monitorfilename:
+								c.appendExten("%s" % obj.pattern, "MixMonitor(%s.%s|%s)" % (self.monitorfilename,self.monitorfileformat,options))
+							else:
+								c.appendExten("%s" % obj.pattern, "MixMonitor(${TIMESTAMP}-${CALLERIDNAME}(${CALLERIDNUM})-${EXTEN}.%s|%s)" % (self.monitorfileformat,options))
+						
+						secret = self.__getitem__("dialout_%s_secret" % obj.name)							
 						if secret:
-							c.append("exten=>%s,1,Macro(%s,${EXTEN},%s,%s)" % (obj.pattern,obj.name,secret,timeoutvalue))	
+							c.appendExten("%s" % obj.pattern,"Macro(%s,%s${EXTEN:%s},%s,%s)" % (obj.name,obj.addprefix,obj.rmprefix,secret,timeoutvalue))
 						else:
-							c.append("exten=>%s,1,Macro(%s,${EXTEN},n,%s)" % (obj.pattern,obj.name,timeoutvalue))	
+							c.appendExten("%s" % obj.pattern,"Macro(%s,%s${EXTEN:%s},n,%s)" % (obj.name,obj.addprefix,obj.rmprefix,timeoutvalue))
 				except KeyError:
 					pass
+			elif obj.__class__.__name__ == 'CfgAppPhoneQuickDial':
+				prefix = obj.dialprefix
+				c.appendExten("_%sXX" % prefix,"Set(dest=${DB(QUICKDIALLIST/${CALLERIDNUM}/${EXTEN:%d})})" % len(prefix), e="Playback(privacy-invalid)")
+				c.appendExten("_%sXX" % prefix,"Goto(${dest},1)")
 	
 	def createPanelConfig(self):
 		try:
@@ -787,6 +1066,17 @@ class CfgApp(Cfg):
 	of software that has a number and that you can dial)."""
 
 	groupName ="Applications"
+
+	def head(self):
+		return (_("Extension"), _("Type"), _("Virtual PBX"))
+
+	def row(self):
+		try:
+			pbx = self.pbx
+		except AttributeError:
+			pbx = _('None')
+		return (self.ext, self.shortName, pbx)
+ 
 	
 class CfgDialout(Cfg):
 	"""Base class for dialout entries."""
