@@ -586,6 +586,7 @@ def connect(username=None,password=None):
 				break
 	if not username and not password:
 		# We couldn't log in
+		print "***** manager cant login"
 		return
 
 	conn = ManagerEvents(username,password)
@@ -616,6 +617,13 @@ def setVar(family, key, val):
 	else:
 		conn.action('Command', Command='database del %s %s' % (family,key))
 
+def getVarFamily(family):
+	varlist = []
+	for s in conn.action('Command', Command='database show %s' % family):
+		if s.startswith("/%s" % family):
+			varlist.append(s[len(family)+2:])
+	return varlist
+
 def checkMailBox(ext):
 	vmstate = {}
 	for s in conn.action('MailboxCount', Mailbox=ext):
